from typing import Any, Dict, Optional


def get_click_command_path(ctx: Any, parsed_options_dict: Optional[Dict[str, str]] = None) -> str:
    cm_path = f"i8 {' '.join(ctx.command_path.split(' ')[1:])}"
    params = ctx.params
    args = []
    options = {}
    for p in ctx.command.params:
        if p.param_type_name == "argument":
            args.append(params[p.name])
        elif p.param_type_name == "option" and params[p.name] is not None:
            options[f"--{p.name}"] = params[p.name] if type(params[p.name]) != bool else None

    if parsed_options_dict:
        options = {**options, **parsed_options_dict}
    return f"{cm_path} {' '.join([f'{k} {val}' if val else k for (k, val) in options.items()])} {''.join(args)}"



from typing import List

import click
import investor8_sdk
from rich.console import Console

from i8_terminal.commands.watchlist import watchlist
from i8_terminal.config import USER_SETTINGS
from i8_terminal.types.ticker_param_type import TickerParamType


def create_watchlist(name: str, tickers: List[str]) -> None:
    investor8_sdk.UserApi().create_watchlist(
        body={"UserId": USER_SETTINGS.get("user_id"), "Name": name, "Tickers": tickers}
    )


@watchlist.command()
@click.option(
    "--name",
    "-n",
    help="Name of the watchlist you want to create.",
)
@click.option("--tickers", "-k", type=TickerParamType(), required=True, help="Comma-separated list of tickers.")
def create(name: str, tickers: str) -> None:
    console = Console()
    tickers_list = tickers.replace(" ", "").upper().split(",")
    with console.status("Creating Watchlist...", spinner="material"):
        create_watchlist(name, tickers_list)
    console.print(f"✅ Watchlist [cyan]{name}[/cyan] is created successfully!")



from rich.console import Console

from i8_terminal.commands.screen import screen


@screen.command()
def list() -> None:
    console = Console()
    console.print("The screen list command is not implemented yet!", style="yellow")



from typing import Any, Dict, List

import dash_bootstrap_components as dbc
from dash import dcc, html
from plotly.graph_objects import Figure

from i8_terminal.common.utils import to_snake_case
from i8_terminal.config import APP_SETTINGS


def get_fig_config(cmd_context: Dict[str, Any]) -> Dict[str, Any]:
    return {"toImageButtonOptions": {"filename": f'{to_snake_case(cmd_context["plot_title"])}'}}


def create_plot_layout(fig: Figure, cmd_context: Dict[str, Any]) -> html.Div:
    fig_config = get_fig_config(cmd_context)
    return html.Div(
        [
            html.Div(dcc.Store(id="cmdContextStore", data=cmd_context)),
            html.Div(dcc.Store(id="figDict", data=fig.to_dict())),
            html.Div(
                [
                    html.Img(src="assets/i8t_logo.png", className="two columns", style={"margin": "auto"}),
                    html.A(
                        html.Button("Save / Publish Plot", id="openModalBtn", n_clicks=0, className="i8-button"),
                        className="two columns",
                        style={"margin": "auto"},
                    ),
                ],
                className="row",
            ),
            html.Div(
                dbc.Alert("", id="saveAlert", color="success", is_open=False),
                className="row",
            ),
            html.Div(
                [
                    html.Div(
                        [dcc.Graph(id="mainPlot", figure=fig, responsive=True, config=fig_config)],
                        className="pretty_container",
                    )
                ],
                className="row",
            ),
            html.Div(
                [
                    html.A("", id="fakeOutput", className="two columns", style={"margin": "auto"}),
                    html.Button(
                        "Close",
                        id="closeServerBtn",
                        n_clicks=0,
                        className="btn-secondary two columns",
                        style={"margin": "auto"},
                    ),
                ],
                className="row",
                style={"margin-top": "30px", "margin-right": "15px"},
            ),
            dbc.Modal(_get_modal_layout(cmd_context), id="savePlotModal", is_open=False, size="lg"),
        ],
        id="mainContainer",
        style={"display": "flex", "flex-direction": "column"},
    )


def _get_modal_layout(cmd_context: Dict[str, Any]) -> List[Any]:
    return [
        dbc.ModalHeader(
            [
                dbc.ModalTitle("Save / Publish plot on your Investoreight profile"),
            ],
            id="modalheader",
        ),
        dbc.ModalBody(
            [
                html.Div(
                    [
                        dbc.Label("Plot Title", className="h4"),
                        dbc.Input(
                            id="titleInput",
                            value=cmd_context["plot_title"],
                            type="text",
                        ),
                    ],
                    className="mb-3",
                ),
                html.Div(
                    [
                        dbc.Label("Notes (Optional)", className="h4"),
                        dcc.Textarea(
                            id="userNotesInput",
                            value="",
                            style={"width": "100%", "height": 100},
                        ),
                    ],
                    className="mb-3",
                ),
                html.Div(
                    [
                        dbc.Checklist(
                            options=[
                                {"label": "Make this plot public (URL can be shared)", "value": 1},
                            ],
                            value=[1],
                            id="isPublicBtn",
                            inline=True,
                            switch=True,
                        ),
                    ],
                ),
            ],
            style={"padding": "2%"},
        ),
        dbc.ModalFooter(
            [
                dcc.Loading(
                    id="loading-2",
                    children=dbc.Alert(
                        "Chart Saved Successfully!", id="save_alert", className="ml-auto", is_open=False
                    ),
                    type="circle",
                    style={"marginRight": 60},
                ),
                dbc.Button("Save Plot", id="saveBtn", n_clicks=0, className="i8_button"),
            ]
        ),
    ]


def get_chart_layout() -> List[Dict[str, Any]]:
    return [
        dict(
            source="assets/i8t_chart_logo.png",
            xref="paper",
            yref="paper",
            x=1.01,
            y=1,
            sizex=0.2,
            sizey=0.2,
            xanchor="right",
            yanchor="bottom",
        )
    ]


def get_date_range(period_code: int) -> Dict[str, List[Dict[str, Any]]]:
    date_range: Dict[str, List[Dict[str, Any]]] = {"buttons": []}
    if period_code >= 2:
        date_range["buttons"].append(dict(count=5, label="5D", step="day", stepmode="backward"))
    elif period_code >= 3:
        date_range["buttons"].append(dict(count=1, label="1M", step="month", stepmode="backward"))
    elif period_code >= 4:
        date_range["buttons"].append(dict(count=3, label="3M", step="month", stepmode="backward"))
    elif period_code >= 5:
        date_range["buttons"].append(dict(count=6, label="6M", step="month", stepmode="backward"))
    elif period_code >= 6:
        date_range["buttons"].append(dict(count=1, label="YTD", step="year", stepmode="todate"))
        date_range["buttons"].append(dict(count=1, label="1Y", step="year", stepmode="backward"))
    elif period_code >= 7:
        date_range["buttons"].append(dict(count=3, label="3Y", step="year", stepmode="backward"))
    elif period_code >= 8:
        date_range["buttons"].append(dict(count=5, label="5Y", step="year", stepmode="backward"))
    date_range["buttons"].append(dict(step="all"))

    return date_range


def get_plot_default_layout() -> Dict[str, Any]:
    return dict(
        images=get_chart_layout(),
        paper_bgcolor=APP_SETTINGS["styles"]["plot"]["default"]["paper_bgcolor"],
        plot_bgcolor=APP_SETTINGS["styles"]["plot"]["default"]["plot_bgcolor"],
    )


def get_terminal_command_layout() -> Dict[str, Any]:
    return dict(color=APP_SETTINGS["styles"]["terminal"]["command"]["color"])



from typing import Any, Dict, List, Optional

import click
import investor8_sdk
import pandas as pd
from investor8_sdk.models.stock_info_master_dto import StockInfoMasterDto
from pandas import DataFrame
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.tree import Tree

from i8_terminal.commands.company import company
from i8_terminal.common.utils import export_data
from i8_terminal.types.ticker_param_type import TickerParamType

from i8_terminal.common.formatting import format_date, format_fyq, format_number  # isort:skip


def get_stock_infos_df(tickers: List[str], exportize: Optional[bool] = False) -> Optional[DataFrame]:
    stock_infos = []
    for ticker in tickers:
        try:
            resp = investor8_sdk.StockInfoApi().get_stock_info_master(ticker=ticker)
            if resp:
                stock_infos.append(resp)
        except Exception:
            continue
    if not stock_infos:
        return None
    return prepare_stock_infos_df(stock_infos, exportize)


def prepare_stock_infos_df(stock_infos: List[StockInfoMasterDto], exportize: Optional[bool]) -> DataFrame:
    available_stock_infos_dict = {
        "summary": {
            "name": {"display_name": "Name", "unit": "str"},
            "exchange": {"display_name": "Exchange", "unit": "str"},
            "sector": {"display_name": "Sector", "unit": "str"},
            "industry_group": {"display_name": "Industry", "unit": "str"},
            "market_cap": {"display_name": "Market Cap", "unit": "usd"},
            "pe_ratio_ttm": {"display_name": "P/E Ratio", "unit": "usdpershare"},
            "current_price": {"display_name": "Current Price", "unit": "usd"},
            "change_perc": {"display_name": "Change", "unit": "percentage"},
        },
        "financials": {
            "fyq": {"display_name": "FYQ", "unit": "fyq"},
            "operating_revenue": {"display_name": "Operating Revenue", "unit": "usd"},
            "total_revenue": {"display_name": "Total Revenue", "unit": "usd"},
            "total_grossprofit": {"display_name": "Total Gross Profit", "unit": "usd"},
            "other_income": {"display_name": "Other Income", "unit": "usd"},
            "net_income": {"display_name": "Net Income", "unit": "usd"},
            "basic_eps": {"display_name": "Basic EPS", "unit": "usdpershare"},
            "diluted_eps": {"display_name": "Diluted EPS", "unit": "usdpershare"},
            "filing_date": {"display_name": "Filing Date", "unit": "date"},
            "adjusted_basic_eps": {"display_name": "Adjusted Basic EPS", "unit": "usd"},
        },
        "price_returns": {
            "OneDay": {"display_name": "One Day", "unit": "percentage"},
            "OneWeek": {"display_name": "One Week", "unit": "percentage"},
            "OneMonth": {"display_name": "One Month", "unit": "percentage"},
            "ThreeMonth": {"display_name": "Three Month", "unit": "percentage"},
            "SixMonth": {"display_name": "Six Month", "unit": "percentage"},
            "YearToDate": {"display_name": "Year To Date", "unit": "percentage"},
            "OneYear": {"display_name": "One Year", "unit": "percentage"},
            "TwoYear": {"display_name": "Two Year", "unit": "percentage"},
            "FiveYear": {"display_name": "Five Year", "unit": "percentage"},
        },
    }
    stock_infos_dict: Dict[str, Dict[str, Any]] = {}
    for stock_info in stock_infos:
        ticker = stock_info.ticker
        stock_infos_dict[ticker] = {}
        for name, value in stock_info.summary.to_dict().items():
            if name in available_stock_infos_dict["summary"]:
                stock_infos_dict[ticker][
                    f"Summary-{available_stock_infos_dict['summary'][name]['display_name']}"
                ] = format_value(
                    value,
                    available_stock_infos_dict["summary"][name]["unit"],
                    exportize,
                    colorize=True if name == "change_perc" else False,
                )
        for name, value in stock_info.latest_financials.quarterly.to_dict().items():
            if name in available_stock_infos_dict["financials"]:
                stock_infos_dict[ticker][
                    f"Financials-{available_stock_infos_dict['financials'][name]['display_name']}"
                ] = format_value(value, available_stock_infos_dict["financials"][name]["unit"], exportize)
        for name, value in stock_info.price_returns.items():
            if name in available_stock_infos_dict["price_returns"]:
                stock_infos_dict[ticker][
                    f"Price Return-{available_stock_infos_dict['price_returns'][name]['display_name']}"
                ] = format_value(
                    value, available_stock_infos_dict["price_returns"][name]["unit"], exportize, colorize=True
                )
    df = DataFrame(stock_infos_dict.values(), index=stock_infos_dict.keys()).T
    df = df.where(pd.notnull(df), "-")
    df.index = df.index.set_names(["Name"])
    df = df.reset_index()
    df["Section"] = df["Name"].apply(lambda name: name.split("-")[0])
    df["Name"] = df["Name"].apply(lambda name: name.split("-")[1])

    return df


def format_value(value: Any, unit: str, exportize: Optional[bool], colorize: Optional[bool] = False) -> Any:
    if value is None:
        return "-"
    if unit == "str":
        return value
    elif unit == "date":
        return format_date(value)
    elif unit == "fyq":
        return format_fyq(value)
    else:
        return format_number(value, unit, humanize=True, exportize=exportize, colorize=colorize)  # type: ignore


def companies_df2tree(df: DataFrame, tickers: List[str]) -> Tree:
    col_width = 40
    plot_title = f"Comparison of {', '.join(tickers)}"
    plot_title = " and ".join(plot_title.rsplit(", ", 1))
    tree = Tree(Panel(plot_title, width=50))
    # Add header table to tree
    header_table = Table(width=50 + (col_width * (len(tickers) - 1)), show_lines=False, show_header=False, box=None)
    header_table.add_column(width=35, style="magenta")
    for p in tickers:
        header_table.add_column(width=col_width, justify="center", style="magenta")
    header_table.add_row("Ticker", *tickers)
    tree.add(header_table)

    for sec_name, sec_values in df.groupby("Section", sort=False):
        sec_branch = tree.add(f"[cyan]{sec_name}")
        for i, r in sec_values.iterrows():
            t = Table(width=46 + (col_width * (len(tickers) - 1)), show_lines=False, show_header=False, box=None)
            t.add_column(width=31)
            for tk in tickers:
                t.add_column(width=col_width, justify="center")
            t.add_row(r["Name"], *[f"{d}" for d in r[tickers].values])
            sec_branch.add(t)

    return tree


@company.command()
@click.option("--tickers", "-k", type=TickerParamType(), required=True, help="Comma-separated list of tickers.")
@click.option("--export", "export_path", "-e", help="Filename to export the output to.")
def compare(tickers: str, export_path: Optional[str]) -> None:
    console = Console()
    tickers_list = tickers.replace(" ", "").upper().split(",")
    with console.status("Fetching data...", spinner="material") as status:
        stock_infos_df = get_stock_infos_df(tickers_list, exportize=True if export_path else False)
        if stock_infos_df is None:
            status.stop()
            click.echo("No data found!")
            return
    if export_path:
        stock_infos_df.drop(columns=["Section"], inplace=True)  # type: ignore
        export_data(stock_infos_df, export_path)
    else:
        tree = companies_df2tree(stock_infos_df, tickers_list)
        console.print(tree)



import click


@click.group()
def cli() -> None:
    """i8 Terminal - Modern Market Research with the Power of Terminal

    Copyright © 2020-2022 Investoreight | https://www.investoreight.com/"""
    pass



from typing import List

from setuptools import find_packages, setup

from i8_terminal.utils import get_version

PACKAGE_NAME = "i8_terminal"


def get_long_description() -> str:
    with open("README.md", "r", encoding="utf-8") as fh:
        long_description = fh.read()
    return long_description


def get_requirements() -> List[str]:
    with open("requirements.txt", "r", encoding="utf-8") as fh:
        requirements = fh.read()
    return requirements.strip().split("\n")


COMMON_ARGS = dict(
    name="i8-terminal",
    version=get_version(),
    author="investoreight",
    author_email="info@investoreight.com",
    license="LICENSE.txt",
    description="Investor8 CLI",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
)


setup(
    **COMMON_ARGS,
    packages=find_packages(),
    package_dir={PACKAGE_NAME: PACKAGE_NAME, "": PACKAGE_NAME},
    install_requires=get_requirements(),
    entry_points={"console_scripts": [f"i8={PACKAGE_NAME}.main:main"]},
    include_package_data=True,
)



from typing import List

import click
import investor8_sdk
from rich.console import Console
from rich.style import Style

from i8_terminal.app.layout import get_terminal_command_layout
from i8_terminal.commands.watchlist import watchlist
from i8_terminal.config import USER_SETTINGS
from i8_terminal.types.ticker_param_type import TickerParamType

from i8_terminal.types.user_watchlists_param_type import UserWatchlistsParamType  # isort:skip


def add_tickers_to_watchlist(name: str, tickers: List[str]) -> None:
    wl = investor8_sdk.UserApi().get_watchlist_by_name_user_id(name=name, user_id=USER_SETTINGS.get("user_id"))
    investor8_sdk.UserApi().add_to_watchlist(body={"Id": wl.id, "Tickers": tickers})


@watchlist.command()
@click.option(
    "--name",
    "-n",
    type=UserWatchlistsParamType(),
    help="Name of the watchlist you want to see more details.",
)
@click.option("--tickers", "-k", type=TickerParamType(), required=True, help="Comma-separated list of tickers.")
def add(name: str, tickers: str) -> None:
    console = Console()
    tickers_list = tickers.replace(" ", "").upper().split(",")
    with console.status("Updating Watchlist...", spinner="material"):
        add_tickers_to_watchlist(name, tickers_list)
    console.print(
        f"✅ Ticker{'s' if len(tickers_list) > 1 else ''} [cyan]{', '.join(tickers_list)}[/cyan] added to watchlist [cyan]{name}[/cyan] successfully!"
    )
    terminal_command_style = Style(**get_terminal_command_layout())
    console.print(
        f'Try `[{terminal_command_style}]watchlist details --name "{name}"[/{terminal_command_style}]` to see the watchlist.'
    )



import enum
from difflib import SequenceMatcher
from typing import Optional

import click
from pandas import DataFrame
from rich.console import Console


class PlotType(enum.Enum):
    CHART = "chart"
    TABLE = "table"


def to_snake_case(value: str) -> str:
    return "_".join(value.lower().split())


def get_period_code(period: str) -> int:
    return {"1D": 1, "5D": 2, "1M": 3, "3M": 4, "6M": 5, "1Y": 6, "3Y": 7, "5Y": 8}.get(period, 3)


def get_period_days(period: str) -> int:
    return {"1D": 1, "5D": 5, "1M": 30, "3M": 90, "6M": 180, "1Y": 365, "3Y": 1095, "5Y": 1825}.get(period, 365)


def similarity(a: str, b: str) -> float:
    return SequenceMatcher(None, a, b).ratio()


def export_data(export_df: DataFrame, export_path: str, index: bool = False) -> None:
    console = Console()
    extension = export_path.split(".")[-1]
    if extension == "csv":
        export_df.to_csv(export_path, index=index)
        console.print(f"Data is saved on: {export_path}")
    elif extension == "xlsx":
        export_df.to_excel(export_path, index=index)
        console.print(f"Data is saved on: {export_path}")
    else:
        console.print("export_path is not valid")


def validate_ticker(ctx: click.Context, param: str, value: str) -> Optional[str]:
    if len(value.replace(" ", "").split(",")) > 1:
        click.echo(click.style(f"`{value}` is not a valid ticker name.", fg="yellow"))
        ctx.exit()
    return value
